// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/samples/signals/perform.c $
// $Rev: 52125 $
// $Date: 2022-12-01 11:46:20 -0600 (Thu, 01 Dec 2022) $

// 24DSI64C200K: Sample Application: source file

#include "main.h"



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32		b_div;	// burst rate divisor
	s32		b_rate;	// burst rate
	time_t	end;
	int		errs		= 0;
	s32		fclk;
	s32		fgen_div	= -1;
	s32		fref;
	double	freq;
	s32		fsamp		= 10000;
	s32		mclk_div	= -1;
	time_t	now;
	s32		nref;
	s32		nvco;
	s32		over_sample	= -1;
	double	rate;

	errs	+= dsi64c200k_initialize		(args->fd, -1, 1);
	errs	+= dsi64c200k_control_mode		(args->fd, -1, 1, DSI64C200K_CONTROL_MODE_INITIATOR, NULL);
	errs	+= dsi64c200k_xcvr_type			(args->fd, -1, 1, args->xcvr_type, NULL);

	// The output clock signal. ===============================================
	gsc_label("Clock Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dsi64c200k_query				(args->fd, -1, 1, DSI64C200K_QUERY_FREF_DEFAULT, &fref);
	errs	+= dsi64c200k_fsamp_ai_compute	(args->fd, -1, 1, fref, fsamp, 0, &nvco, &nref, &fgen_div, &mclk_div, &over_sample, &rate);
	errs	+= dsi64c200k_nvco				(args->fd, -1, 1, 25, NULL);
	errs	+= dsi64c200k_nref				(args->fd, -1, 1, 20, NULL);
	errs	+= dsi64c200k_fgen_div			(args->fd, -1, 1, DSI64C200K_FGEN_DIV_1, NULL);
	errs	+= dsi64c200k_mclk_div			(args->fd, -1, 1, DSI64C200K_MCLK_DIV_32_ECO, NULL);
	errs	+= dsi64c200k_over_sample		(args->fd, -1, 1, DSI64C200K_OVER_SAMPLE_64, NULL);
	errs	+= dsi64c200k_clk_src			(args->fd, -1, 1, DSI64C200K_CLK_SRC_RATE_GEN, NULL);
	errs	+= dsi64c200k_ext_clk_src		(args->fd, -1, 1, DSI64C200K_EXT_CLK_SRC_MCLK, NULL);

	gsc_label("Signal Frequency");
	freq	= (double) fref * nvco / nref / 2;
	printf("~");
	gsc_label_float_comma(freq, 1, 0);
	printf(" Hz\n");

	gsc_label_level_dec();

	// The output sync signal. ================================================
	gsc_label("Sync Output");
	printf("\n");
	gsc_label_level_inc();
	errs	+= dsi64c200k_query(args->fd, -1, 1, DSI64C200K_QUERY_MASTER_CLOCK, &fclk);

	gsc_label("Burst Rate");
	b_rate	= fsamp / 4;
	printf("%ld Triggers/S\n", (long) b_rate);

	b_div	= fclk / b_rate;
	errs	+= dsi64c200k_burst_rate_div	(args->fd, -1, 1, b_div, NULL);

	errs	+= dsi64c200k_burst_size		(args->fd, -1, 1000, 1, NULL);
	errs	+= dsi64c200k_burst_timer		(args->fd, -1, 1, DSI64C200K_BURST_TIMER_ENABLE, NULL);
	errs	+= dsi64c200k_burst_enable		(args->fd, -1, 1, DSI64C200K_BURST_ENABLE_YES, NULL);

	gsc_label("Signal Frequency");
	gsc_label_long_comma(b_rate);
	printf(" Pulses/S\n");

	gsc_label_level_dec();

	// Waiting to stop. =======================================================
	gsc_label("Working");
	printf("%d seconds ... ", args->seconds);
	fflush(stdout);
	end	= time(NULL) + args->seconds;

	for (; errs == 0;)
	{
		now	= time(NULL);

		if (now > end)
			break;

		os_sleep_ms(100);
	}

	printf("Done\n");
	gsc_label_level_dec();

	return(errs);
}



