// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_ai_buf_thr_sts.c $
// $Rev: 52653 $
// $Date: 2023-03-21 10:02:20 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi64c200k_ai_buf_thr_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI64C200K_IOCTL_AI_BUF_THR_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi64c200k_ai_buf_thr_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("AI Buffer Threshold Status", index);

	ret		= dsi64c200k_ioctl(fd, DSI64C200K_IOCTL_AI_BUF_THR_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI64C200K_AI_BUF_THR_STS_IDLE:

			ptr	= "Idle";
			break;

		case DSI64C200K_AI_BUF_THR_STS_ACTIVE:

			ptr	= "Active";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


