// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_config_ai.c $
// $Rev: 52116 $
// $Date: 2022-12-01 11:05:52 -0600 (Thu, 01 Dec 2022) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi64c200k_config_ai
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fref	This is the PLL Fref value, or -1 to use the default.
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	dsi64c200k_config_ai(int fd, int index, int verbose, s32 fref, s32 fsamp)
{
	int		errs		= 0;
	s32		fgen_div	= -1;	// auto-select
	s32		mclk_div	= DSI64C200K_MCLK_DIV_4_FAST;	// use -1 for auto-select
	s32		nref;
	s32		nvco;
	int		optimal		= 0;
	s32		over_sample	= -1;	// auto-select
	double	rate;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= dsi64c200k_initialize			(fd, index, verbose);
	errs	+= dsi64c200k_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= dsi64c200k_rx_io_overflow		(fd, index, verbose, DSI64C200K_IO_OVERFLOW_CHECK,		NULL);
	errs	+= dsi64c200k_rx_io_timeout			(fd, index, verbose, 30,								NULL);
	errs	+= dsi64c200k_rx_io_underflow		(fd, index, verbose, DSI64C200K_IO_UNDERFLOW_CHECK,		NULL);
	errs	+= dsi64c200k_ai_mode				(fd, index, verbose, DSI64C200K_AI_MODE_DIFF,			NULL);
	errs	+= dsi64c200k_ai_buf_thresh			(fd, index, verbose, 128L * 1024,						NULL);
	errs	+= dsi64c200k_ai_filter				(fd, index, verbose, DSI64C200K_AI_FILTER_LOW_LATENCY,	NULL);
	errs	+= dsi64c200k_chan_grp_active		(fd, index, verbose, 0xFF,								NULL);
	errs	+= dsi64c200k_sw_sync_mode			(fd, index, verbose, DSI64C200K_SW_SYNC_MODE_CLR_BUF,	NULL);
	errs	+= dsi64c200k_data_format			(fd, index, verbose, DSI64C200K_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= dsi64c200k_data_width			(fd, index, verbose, DSI64C200K_DATA_WIDTH_16,			NULL);
	errs	+= dsi64c200k_dio_dir_out			(fd, index, verbose, 0x0,								NULL);
	errs	+= dsi64c200k_control_mode			(fd, index, verbose, DSI64C200K_CONTROL_MODE_INITIATOR,	NULL);
	errs	+= dsi64c200k_ext_clk_src			(fd, index, verbose, DSI64C200K_EXT_CLK_SRC_GEN,		NULL);
	errs	+= dsi64c200k_irq_sel				(fd, index, verbose, DSI64C200K_IRQ_INIT_DONE,			NULL);
	errs	+= dsi64c200k_xcvr_type				(fd, index, verbose, DSI64C200K_XCVR_TYPE_LVDS,			NULL);
	errs	+= dsi64c200k_aux_clock				(fd, index, verbose, DSI64C200K_AUX_CLOCK_INACTIVE,		NULL);
	errs	+= dsi64c200k_aux_sync				(fd, index, verbose, DSI64C200K_AUX_SYNC_INACTIVE,		NULL);
	errs	+= dsi64c200k_burst_enable			(fd, index, verbose, DSI64C200K_BURST_ENABLE_NO,		NULL);

	errs	+= dsi64c200k_query					(fd, index, verbose, DSI64C200K_QUERY_FREF_DEFAULT, &fref);
	errs	+= dsi64c200k_fsamp_ai_compute		(fd, index, verbose, fref, fsamp, optimal, &nvco, &nref, &fgen_div, &mclk_div, &over_sample, &rate);
	errs	+= dsi64c200k_nvco					(fd, index, verbose, nvco,								NULL);
	errs	+= dsi64c200k_nref					(fd, index, verbose, nref,								NULL);
	errs	+= dsi64c200k_fgen_div				(fd, index, verbose, fgen_div,							NULL);
	errs	+= dsi64c200k_mclk_div				(fd, index, verbose, mclk_div,							NULL);
	errs	+= dsi64c200k_over_sample			(fd, index, verbose, over_sample,						NULL);

	errs	+= dsi64c200k_ai_buf_enable			(fd, index, verbose, DSI64C200K_AI_BUF_ENABLE_YES,		NULL);
	errs	+= dsi64c200k_channels_ready_wait	(fd, index, verbose,									NULL);
	errs	+= dsi64c200k_fsamp_ai_report_all	(fd, index, verbose,									NULL);

	errs	+= dsi64c200k_auto_calibrate		(fd, index, verbose);
	errs	+= dsi64c200k_auto_cal_sts			(fd, index, verbose,									NULL);
	errs	+= dsi64c200k_ai_buf_clear			(fd, index, verbose);
	errs	+= dsi64c200k_ai_buf_overflow		(fd, index, verbose, DSI64C200K_AI_BUF_OVERFLOW_CLEAR,	NULL);
	errs	+= dsi64c200k_ai_buf_underflow		(fd, index, verbose, DSI64C200K_AI_BUF_UNDERFLOW_CLEAR,	NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


