// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_reg.c $
// $Rev: 52116 $
// $Date: 2022-12-01 11:05:52 -0600 (Thu, 01 Dec 2022) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)			"GSC " #a, DSI64C200K_GSC_##a
#define	NAME_WIDTH			24

#define	FIELD_SHOW(beg,end,eol,lst,dsc)		\
			gsc_reg_field_show(width+10,NAME_WIDTH,value,(beg),(end),(eol),(lst),(dsc))



//*****************************************************************************
static int _decode_bctlr(int fd, int supported, u32 value, int width)
{
	static const char*	_aim[]	=
	{
		"Differential",
		"Reserved",
		"Zero Test",
		"Vref Test"
	};

	static const char*	_irq[]	=
	{
		"Initialization Complete",
		"Auto-Calibration Complete",
		"Channels Ready Low-to-High",
		"Threshold Flag Low-to-High",
		"Threshold Flag High-to-Low",
		"Burst Complete",
		"Channels Ready High-to-Low",
		"Reserved"
	};

	static const char*	_control[]	= { "Target",			"Initiator"			};
	static const char*	_disable[]	= { "Disabled",			"Enabled"			};
	static const char*	_extern[]	= { "Mclk",				"Rate generator"	};
	static const char*	_fail[]		= { "Fail",				"Pass"				};
	static const char*	_format[]	= { "Twos Compliment",	"Offset Binary"		};
	static const char*	_idle[]		= { "Idle",				"Active"			};
	static const char*	_not_rdy[]	= { "Not Ready",		"Ready"				};
	static const char*	_sync[]		= { "Sync",				"Clear Buffer"		};
	static const char*	_xcvr[]		= { "LVDS",				"TTL"				};

	FIELD_SHOW( 1,  0, 1, _aim,		"Analog Input Mode"		);
	FIELD_SHOW( 3,  2, 1, NULL,		"Reserved"				);
	FIELD_SHOW( 4,  4, 1, _format,	"Data Format"			);
	FIELD_SHOW( 5,  5, 1, _control,	"Control Mode"			);
	FIELD_SHOW( 6,  6, 1, _idle,	"SW Sync"				);
	FIELD_SHOW( 7,  7, 1, _idle,	"Auto-Calibration"		);
	FIELD_SHOW(10,  8, 1, _irq,		"IRQ Select"			);
	FIELD_SHOW(11, 11, 1, _idle,	"IRQ Request"			);
	FIELD_SHOW(12, 12, 1, _fail,	"Auto-Cal Status"		);
	FIELD_SHOW(13, 13, 1, _not_rdy,	"Channels Ready"		);
	FIELD_SHOW(14, 14, 1, _idle,	"Buffer Threshold Flag"	);
	FIELD_SHOW(15, 15, 1, _idle,	"Initialize"			);
	FIELD_SHOW(16, 16, 1, _disable,	"Burst Trigger Timer"	);
	FIELD_SHOW(17, 17, 1, _sync,	"Sync Mode"				);
	FIELD_SHOW(18, 18, 1, _extern,	"External Clock Source"	);
	FIELD_SHOW(19, 19, 1, _idle,	"Initialize ADCs"		);
	FIELD_SHOW(20, 20, 1, NULL,		"Reserved"				);
	FIELD_SHOW(21, 21, 1, _xcvr,	"Transceivers"			);
	FIELD_SHOW(22, 22, 1, _disable,	"Bursting"				);
	FIELD_SHOW(23, 23, 1, _idle,	"SW Burst Trigger"		);
	FIELD_SHOW(31, 23, 1, NULL,		"Reserved"				);

	return(0);
}



//*****************************************************************************
static int _decode_rcr(int fd, int supported, u32 value, int width)
{
	FIELD_SHOW(11,  0, 1, NULL,	"Nvco"		);
	FIELD_SHOW(15, 12, 1, NULL,	"Reserved"	);
	FIELD_SHOW(27, 16, 1, NULL,	"Nref"		);
	FIELD_SHOW(31, 28, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _decode_diopr(int fd, int supported, u32 value, int width)
{
	static const char*	_in[]	= { "Input",	"Output"	};
	static const char*	_low[]	= { "Low (0)",	"High (1)"	};

	FIELD_SHOW( 0,  0, 1, _low,	"Digital Data D0"		);
	FIELD_SHOW( 1,  1, 1, _low,	"Digital Data D1"		);
	FIELD_SHOW( 2,  2, 1, _low,	"Digital Data D2"		);
	FIELD_SHOW( 3,  3, 1, _low,	"Digital Data D3"		);
	FIELD_SHOW( 7,  4, 1, NULL,	"Reserved"				);
	FIELD_SHOW( 8,  8, 1, _in,	"Digital Direction D0"	);
	FIELD_SHOW( 9,  9, 1, _in,	"Digital Direction D1"	);
	FIELD_SHOW(10, 10, 1, _in,	"Digital Direction D2"	);
	FIELD_SHOW(11, 11, 1, _in,	"Digital Direction D3"	);
	FIELD_SHOW(31, 12, 1, NULL,	"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _decode_aicr(int fd, int supported, u32 value, int width)
{
	static const char*	_fgen[]	=
	{
		"/ 1",
		"/ 2",
		"/ 4",
		"/ 8",
		"/ 16",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_mclk[]	=
	{
		"/ 32, 'Eco' Mode",
		"/ 8, 'Median' Mode",
		"/ 4, 'Fast' Mode",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_over[]	=
	{
		"32",
		"64",
		"128",
		"1024",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_clk[]	=
	{
		"Rate Generator",
		"External Fgen",
		"External Mclk",
		"Reserved"
	};

	static const char*	_disable[]	= { "Disabled",			"Enabled"		};
	static const char*	_filter[]	= { "Low-Ripple",		"Low Latency"	};
	static const char*	_sync[]		= { "Internal, BCR",	"External, BCR"	};

	FIELD_SHOW( 0,  0, 1, _disable,	"Group 0 State"			);
	FIELD_SHOW( 1,  1, 1, _disable,	"Group 1 State"			);
	FIELD_SHOW( 2,  2, 1, _disable,	"Group 2 State"			);
	FIELD_SHOW( 3,  3, 1, _disable,	"Group 3 State"			);
	FIELD_SHOW( 4,  4, 1, _disable,	"Group 4 State"			);
	FIELD_SHOW( 5,  5, 1, _disable,	"Group 5 State"			);
	FIELD_SHOW( 6,  6, 1, _disable,	"Group 6 State"			);
	FIELD_SHOW( 7,  7, 1, _disable,	"Group 7 State"			);
	FIELD_SHOW(11,  8, 1, _fgen,	"Fgen Divisor"			);
	FIELD_SHOW(15, 12, 1, _mclk,	"Mclk Divisor"			);
	FIELD_SHOW(19, 16, 1, _over,	"Over Sampling Factor"	);
	FIELD_SHOW(21, 20, 1, _clk,		"Clock Source"			);
	FIELD_SHOW(22, 22, 1, _sync,	"SYNC Source"			);
	FIELD_SHOW(23, 23, 1, _filter,	"Filter"				);
	FIELD_SHOW(31, 24, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _decode_bbsr(int fd, int supported, u32 value, int width)
{
	FIELD_SHOW(23,  0, 1, NULL,	"Size"		);
	FIELD_SHOW(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _decode_bttr(int fd, int supported, u32 value, int width)
{
	FIELD_SHOW(23,  0, 1, NULL,	"Divisor"	);
	FIELD_SHOW(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _decode_bufcr(int fd, int supported, u32 value, int width)
{
	static const char*	_width[]	=
	{
		"16-bits",
		"18-bits",
		"20-bits",
		"24-bits"
	};

	static const char*	_disable[]	= { "No, Disabled",	"Yes, Enabled"	};
	static const char*	_idle[]		= { "Idle",			"Active"		};
	static const char*	_no[]		= { "No",			"Yes"			};

	FIELD_SHOW(18,  0, 1, NULL,		"Threshold"			);
	FIELD_SHOW(19, 19, 1, _disable,	"Buffer Enable"		);
	FIELD_SHOW(20, 20, 1, _idle,	"Clear Buffer"		);
	FIELD_SHOW(22, 21, 1, _width,	"Data Width"		);
	FIELD_SHOW(23, 23, 1, NULL,		"Reserved"			);
	FIELD_SHOW(24, 24, 1, _no,		"Buffer Overflow"	);
	FIELD_SHOW(25, 25, 1, _no,		"Buffer Underflow"	);
	FIELD_SHOW(31, 26, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _decode_bcgfr(int fd, int supported, u32 value, int width)
{
	static const char*	_chans[]	=
	{
		"64 Channels",
		"48 Channels",
		"32 Channels",
		"Reserved"
	};

	static const char*	_filter[]	=
	{
		"900 KHz",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_master[]	=
	{
		"49.152 MHz",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_range[]	=
	{
		"+-10V",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	u32	field;

	FIELD_SHOW(11,  0, 0, NULL,		"Firmware Revision"	);
	field	= GSC_FIELD_DECODE(value, 11, 0);
	printf("%03lX\n", (long) field);

	FIELD_SHOW(15, 12, 1, NULL,		"Reserved"			);
	FIELD_SHOW(17, 16, 1, _chans,	"Channels"			);
	FIELD_SHOW(19, 18, 1, _range,	"Range"				);
	FIELD_SHOW(21, 20, 1, _filter,	"Filter Frequency"	);
	FIELD_SHOW(23, 22, 1, _master,	"Master Clock"		);
	FIELD_SHOW(31, 24, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _decode_bufsr(int fd, int supported, u32 value, int width)
{
	u32	field;

	FIELD_SHOW(18,  0, 0, NULL,	"Buffer Fill Level"	);
	field	= GSC_FIELD_DECODE(value, 19, 0);
	printf("%ld samples\n", (long) field);

	FIELD_SHOW(31, 19, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _decode_idbr(int fd, int supported, u32 value, int width)
{
	u32	field;

	FIELD_SHOW(23,  0, 0, NULL,	"Data & Padding"	);
	field	= GSC_FIELD_DECODE(value, 23, 0);
	printf("0x%06lX\n", (long) field);

	FIELD_SHOW(29, 24, 0, NULL,	"Channel Tag"		);
	field	= GSC_FIELD_DECODE(value, 29, 24);
	printf("0x%02lX\n", (long) field);

	FIELD_SHOW(31, 30, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _decode_asiocr(int fd, int supported, u32 value, int width)
{
	static const char*	_mode[]	=
	{
		"Inactive, Disabled",
		"Input",
		"Output",
		"Reserved"
	};

	FIELD_SHOW( 1,  0, 1, _mode,	"Aux Clock Mode"	);
	FIELD_SHOW( 3,  2, 1, _mode,	"Aux SYNC Mode"		);
	FIELD_SHOW(31,  4, 1, NULL,		"RESERVED"			);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	// name	reg			err	value	ask_support	decode			desc
	{ _GSC_REG(BCTLR),	0,	0,		0,			_decode_bctlr,	"Board Control Register"		},
	{ _GSC_REG(RCR),	0,	0,		0,			_decode_rcr,	"Rate Control Register"			},
	{ _GSC_REG(DIOPR),	0,	0,		0,			_decode_diopr,	"Digital I/O Port Register"		},
	{ _GSC_REG(AICR),	0,	0,		0,			_decode_aicr,	"Analog Input Config Register"	},
	{ _GSC_REG(BBSR),	0,	0,		0,			_decode_bbsr,	"Burst Block Size Register"		},
	{ _GSC_REG(BTTR),	0,	0,		0,			_decode_bttr,	"Burst Trigger Timer Register"	},
	{ _GSC_REG(BUFCR),	0,	0,		0,			_decode_bufcr,	"Buffer Control Register"		},
	{ _GSC_REG(BCFGR),	0,	0,		0,			_decode_bcgfr,	"Board Config Register"			},
	{ _GSC_REG(BUFSR),	0,	0,		0,			_decode_bufsr,	"Buffer Size Register"			},
	{ _GSC_REG(AVR),	0,	0,		0,			NULL,			"Auto-Cal Values Register"		},
	{ _GSC_REG(IDBR),	0,	0,		0,			_decode_idbr,	"Input Data Buffer Register"	},
	{ _GSC_REG(ASIOCR),	0,	0,		0,			_decode_asiocr,	"Aux SYNC I/O Control Register"	},
	{ NULL }
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	dsi64c200k_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dsi64c200k_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	dsi64c200k_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dsi64c200k_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	dsi64c200k_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dsi64c200k_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	dsi64c200k_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dsi64c200k_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	dsi64c200k_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dsi64c200k_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, dsi64c200k_reg_read);
	return(errs);
}



